% Testing Data %
m = 20;
N = 80;
sparsity = 4;
amplitude_max = 100;
A = randn(m, N);
x = zeros(N, 1);
x_index = randsample(N, sparsity)
x_value = randsample(amplitude_max, sparsity)
x(x_index) = x_value;
y = A*x;
eta = 0.05; % for qcbp
gamma = 30; % for bp denoising
y_noise = y + wgn(m, 1, eta^2); % for qcbp and bp denoising

algorithm = 'ADMM';
task = 'bp';
% Task Setting %
if strcmp(algorithm, 'primal-dual')
    switch task
        case 'bp'
            ProxF_conj = @(x,sigma,y)x-sigma*y;
            ProxG = @(x,tau)perform_soft_thresholding(x,tau);
            Cal_Primal_Dual_Gap = @(x, xi, y)norm(x,1)+sum(y.*xi);
        case 'qcbp'
            ProxF_conj = @(x,sigma,y)max(0, 1-eta*sigma/norm(x-sigma*y))*(x-sigma*y);
            ProxG = @(x,tau)perform_soft_thresholding(x,tau);
            Cal_Primal_Dual_Gap = @(x, xi, y)norm(x,1)+sum(y.*xi)+eta*norm(xi);
        case 'bp denoising'
            ProxF_conj = @(x,sigma,y)gamma/(gamma+sigma)*(x-sigma*y);
            ProxG = @(x,tau)perform_soft_thresholding(x,tau);
            Cal_Primal_Dual_Gap = @(x, xi, y)gamma/2*norm(A*x-y)^2+norm(x,1)+sum(y.*xi)+norm(xi)^2/(2*gamma);
    end
end
if strcmp(algorithm, 'dr')
    switch task
        case 'bp'
            ProxF = @(tau, x, y)x+A'/(A*A')*(y-A*x);
            ProxG = @(tau, x)perform_soft_thresholding(x, tau);
    end
end
if strcmp(algorithm, 'ADMM')
    switch task
        case 'bp'
            ProxF = @(tau, x)x-A'*inv(A*A')*(A*x-y);
            ProxG = @(tau, x)perform_soft_thresholding(x, tau);
        case 'bp denoising'
            ProxF = @(tau, x)(A'*A+eye(N)/tau)\(A'*y_noise+x/tau);
            ProxG = @(tau, x)perform_soft_thresholding(x, tau);
    end
end
if strcmp(algorithm, 'APGM')
    switch task
        case 'bp denoising'
            GradF = @(x)A'*(A*x-y_noise);
            ProxG = @(tau, x)perform_soft_thresholding(x, tau);
    end
end

% Perform Task %
if strcmp(algorithm, 'primal-dual')
    switch task
        case 'bp'
            x = primal_dual(A, y, ProxF_conj, ProxG, Cal_Primal_Dual_Gap);
            [value, index] = maxk(x, sparsity)
            x = (A(:,index)'*A(:,index))\A(:,index)'*y
        case 'qcbp'
            x = primal_dual(A, y_noise, ProxF_conj, ProxG, Cal_Primal_Dual_Gap);
            [value, index] = maxk(x, sparsity)
            x = (A(:,index)'*A(:,index))\A(:,index)'*y
        case 'bp denoising'
            x = primal_dual(A, y, ProxF_conj, ProxG, Cal_Primal_Dual_Gap);
            [value, index] = maxk(x, sparsity)
    end
end
if strcmp(algorithm, 'dr')
    switch task
        case 'bp'
            x = Douglas_Rachford(N, y, ProxF, ProxG);
            [value, index] = maxk(x, sparsity)
            x = (A(:,index)'*A(:,index))\A(:,index)'*y
    end
end
if strcmp(algorithm, 'ADMM')
    switch task
        case 'bp'
            x = ADMM(N, eye(N), ProxF, ProxG);
            [value, index] = maxk(x, sparsity)
            x = (A(:,index)'*A(:,index))\A(:,index)'*y 
        case 'bp denoising'
            x = ADMM(N, eye(N), ProxF, ProxG);
            [value, index] = maxk(x, sparsity)
            x = (A(:,index)'*A(:,index))\A(:,index)'*y
    end
end
if strcmp(algorithm, 'APGM')
    switch task
        case 'bp denoising'
            x = APGM(N, norm(A)^2, GradF, ProxG);
            [value, index] = maxk(x, sparsity)
    end
end